@@ -7,6 +7,8 @@ import asyncio
 import concurrent.futures
 
 from typing import Optional, Dict, Tuple, List, IO, Union
+
+import requests
 from pandas import DataFrame
 from requests import Session
 from requests.models import Response
@@ -14,8 +16,10 @@ from requests.exceptions import ConnectionError
 from requests.adapters import HTTPAdapter
 from urllib3.util import Retry
 from tenacity import retry, wait_exponential, stop_after_attempt
+from time import sleep
 
 from vespa.io import VespaQueryResponse, VespaResponse
+from vespa.ml import TextTask
 from vespa.query import QueryModel
 from vespa.evaluation import EvalMetric
 from vespa.package import ApplicationPackage
@@ -143,17 +147,19 @@ class Vespa(object):
             app=self, connections=connections, total_timeout=total_timeout
         )
 
-    def _run_coroutine_new_event_loop(self, loop, coro):
+    @staticmethod
+    def _run_coroutine_new_event_loop(loop, coro):
         asyncio.set_event_loop(loop)
         return loop.run_until_complete(coro)
 
-    def _check_for_running_loop_and_run_coroutine(self, coro):
+    @staticmethod
+    def _check_for_running_loop_and_run_coroutine(coro):
         try:
             _ = asyncio.get_running_loop()
             new_loop = asyncio.new_event_loop()
             with concurrent.futures.ThreadPoolExecutor() as executor:
                 future = executor.submit(
-                    self._run_coroutine_new_event_loop, new_loop, coro
+                    Vespa._run_coroutine_new_event_loop, new_loop, coro
                 )
                 return_value = future.result()
                 return return_value
@@ -189,14 +195,44 @@ class Vespa(object):
 
         return schema.name
 
+    def wait_for_application_up(self, max_wait):
+        """
+        Wait for application ready.
+
+        :param max_wait: Seconds to wait for the application endpoint
+        :return:
+        """
+        try_interval = 5
+        waited = 0
+        while not self.get_application_status() and (waited < max_wait):
+            print(
+                "Waiting for application status, {0}/{1} seconds...".format(
+                    waited, max_wait
+                ),
+                file=self.output_file,
+            )
+            sleep(try_interval)
+            waited += try_interval
+        if waited >= max_wait:
+            raise RuntimeError(
+                "Application did not start, waited for {0} seconds.".format(max_wait)
+            )
+
     def get_application_status(self) -> Optional[Response]:
         """
         Get application status.
 
         :return:
         """
-        with VespaSync(self) as sync_app:
-            return sync_app.get_application_status()
+        endpoint = "{}/ApplicationStatus".format(self.end_point)
+        try:
+            if self.key:
+                response = requests.get(endpoint, cert=(self.cert, self.key))
+            else:
+                response = requests.get(endpoint, cert=self.cert)
+        except ConnectionError:
+            response = None
+        return response
 
     def get_model_endpoint(self, model_id: Optional[str] = None) -> Optional[Response]:
         """Get model evaluation endpoints."""
@@ -204,7 +240,7 @@ class Vespa(object):
         with VespaSync(self) as sync_app:
             return sync_app.get_model_endpoint(model_id=model_id)
 
-    def _build_query_body(
+    def build_query_body(
         self,
         query: Optional[str] = None,
         query_model: Optional[QueryModel] = None,
@@ -361,7 +397,7 @@ class Vespa(object):
                 total_timeout=total_timeout,
                 **kwargs,
             )
-            return self._check_for_running_loop_and_run_coroutine(coro=coro)
+            return Vespa._check_for_running_loop_and_run_coroutine(coro=coro)
         else:
             return self._query_batch_sync(
                 body_batch=body_batch,
@@ -371,33 +407,45 @@ class Vespa(object):
                 **kwargs,
             )
 
-    def feed_data_point(self, schema: str, data_id: str, fields: Dict) -> VespaResponse:
+    def feed_data_point(
+        self, schema: str, data_id: str, fields: Dict, namespace: str = None
+    ) -> VespaResponse:
         """
         Feed a data point to a Vespa app.
 
         :param schema: The schema that we are sending data to.
         :param data_id: Unique id associated with this data point.
         :param fields: Dict containing all the fields required by the `schema`.
+        :param namespace: The namespace that we are sending data to.
         :return: Response of the HTTP POST request.
         """
+        if not namespace:
+            namespace = schema
+
         with VespaSync(app=self) as sync_app:
             return sync_app.feed_data_point(
-                schema=schema, data_id=data_id, fields=fields
+                schema=schema, data_id=data_id, fields=fields, namespace=namespace
             )
 
-    def _feed_batch_sync(self, schema: str, batch: List[Dict]):
+    def _feed_batch_sync(
+        self, schema: str, batch: List[Dict], namespace: str
+    ) -> VespaResponse:
         return [
-            self.feed_data_point(schema, data_point["id"], data_point["fields"])
+            self.feed_data_point(
+                schema, data_point["id"], data_point["fields"], namespace
+            )
             for data_point in batch
         ]
 
     async def _feed_batch_async(
-        self, schema: str, batch: List[Dict], connections, total_timeout
+        self, schema: str, batch: List[Dict], connections, total_timeout, namespace: str
     ):
         async with VespaAsync(
             app=self, connections=connections, total_timeout=total_timeout
         ) as async_app:
-            return await async_app.feed_batch(schema=schema, batch=batch)
+            return await async_app.feed_batch(
+                schema=schema, batch=batch, namespace=namespace
+            )
 
     def feed_batch(
         self,
@@ -406,6 +454,7 @@ class Vespa(object):
         asynchronous=True,
         connections: Optional[int] = 100,
         total_timeout: int = 100,
+        namespace: Optional[str] = None,
     ):
         """
         Feed a batch of data to a Vespa app.
@@ -416,6 +465,7 @@ class Vespa(object):
         :param asynchronous: Set True to send data in async mode. Default to True.
         :param connections: Number of allowed concurrent connections, valid only if `asynchronous=True`.
         :param total_timeout: Total timeout in secs for each of the concurrent requests when using `asynchronous=True`.
+        :param namespace: The namespace that we are sending data to. If no namespace is provided the schema is used.
         :return: List of HTTP POST responses
         """
         if not schema:
@@ -426,16 +476,22 @@ class Vespa(object):
                     "Not possible to infer schema name. Specify schema parameter."
                 )
 
+        if not namespace:
+            namespace = schema
+
         if asynchronous:
             coro = self._feed_batch_async(
                 schema=schema,
                 batch=batch,
                 connections=connections,
                 total_timeout=total_timeout,
+                namespace=namespace,
             )
-            return self._check_for_running_loop_and_run_coroutine(coro=coro)
+            return Vespa._check_for_running_loop_and_run_coroutine(coro=coro)
         else:
-            return self._feed_batch_sync(schema=schema, batch=batch)
+            return self._feed_batch_sync(
+                schema=schema, batch=batch, namespace=namespace
+            )
 
     def feed_df(self, df: DataFrame, include_id: bool = True, **kwargs):
         """
@@ -449,27 +505,40 @@ class Vespa(object):
         batch = parse_feed_df(df=df, include_id=include_id)
         return self.feed_batch(batch=batch, **kwargs)
 
-    def delete_data(self, schema: str, data_id: str) -> VespaResponse:
+    def delete_data(
+        self, schema: str, data_id: str, namespace: str = None
+    ) -> VespaResponse:
         """
         Delete a data point from a Vespa app.
 
         :param schema: The schema that we are deleting data from.
         :param data_id: Unique id associated with this data point.
+        :param namespace: The namespace that we are deleting data from. If no namespace is provided the schema is used.
         :return: Response of the HTTP DELETE request.
         """
-        with VespaSync(self) as sync_app:
-            return sync_app.delete_data(schema=schema, data_id=data_id)
+        if not namespace:
+            namespace = schema
 
-    def _delete_batch_sync(self, schema: str, batch: List[Dict]):
-        return [self.delete_data(schema, data_point["id"]) for data_point in batch]
+        with VespaSync(self) as sync_app:
+            return sync_app.delete_data(
+                schema=schema, data_id=data_id, namespace=namespace
+            )
+
+    def _delete_batch_sync(self, schema: str, batch: List[Dict], namespace: str):
+        return [
+            self.delete_data(schema, data_point["id"], namespace)
+            for data_point in batch
+        ]
 
     async def _delete_batch_async(
-        self, schema: str, batch: List[Dict], connections, total_timeout
+        self, schema: str, batch: List[Dict], connections, total_timeout, namespace: str
     ):
         async with VespaAsync(
             app=self, connections=connections, total_timeout=total_timeout
         ) as async_app:
-            return await async_app.delete_batch(schema=schema, batch=batch)
+            return await async_app.delete_batch(
+                schema=schema, batch=batch, namespace=namespace
+            )
 
     def delete_batch(
         self,
@@ -478,6 +547,7 @@ class Vespa(object):
         asynchronous=True,
         connections: Optional[int] = 100,
         total_timeout: int = 100,
+        namespace: Optional[str] = None,
     ):
         """
         Delete a batch of data from a Vespa app.
@@ -488,6 +558,7 @@ class Vespa(object):
         :param asynchronous: Set True to get data in async mode. Default to True.
         :param connections: Number of allowed concurrent connections, valid only if `asynchronous=True`.
         :param total_timeout: Total timeout in secs for each of the concurrent requests when using `asynchronous=True`.
+        :param namespace: The namespace that we are deleting data from. If no namespace is provided the schema is used.
         :return: List of HTTP POST responses
         """
         if not schema:
@@ -498,51 +569,77 @@ class Vespa(object):
                     "Not possible to infer schema name. Specify schema parameter."
                 )
 
+        if not namespace:
+            namespace = schema
+
         if asynchronous:
             coro = self._delete_batch_async(
                 schema=schema,
                 batch=batch,
                 connections=connections,
                 total_timeout=total_timeout,
+                namespace=namespace,
             )
-            return self._check_for_running_loop_and_run_coroutine(coro=coro)
+            return Vespa._check_for_running_loop_and_run_coroutine(coro=coro)
         else:
-            return self._delete_batch_sync(schema=schema, batch=batch)
+            return self._delete_batch_sync(
+                schema=schema, batch=batch, namespace=namespace
+            )
 
-    def delete_all_docs(self, content_cluster_name: str, schema: str) -> Response:
+    def delete_all_docs(
+        self, content_cluster_name: str, schema: str, namespace: str = None
+    ) -> Response:
         """
         Delete all documents associated with the schema
 
         :param content_cluster_name: Name of content cluster to GET from, or visit.
         :param schema: The schema that we are deleting data from.
+        :param namespace: The  namespace that we are deleting data from. If no namespace is provided the schema is used.
         :return: Response of the HTTP DELETE request.
         """
+        if not namespace:
+            namespace = schema
+
         with VespaSync(self) as sync_app:
             return sync_app.delete_all_docs(
-                content_cluster_name=content_cluster_name, schema=schema
+                content_cluster_name=content_cluster_name,
+                namespace=namespace,
+                schema=schema,
             )
 
-    def get_data(self, schema: str, data_id: str) -> VespaResponse:
+    def get_data(
+        self, schema: str, data_id: str, namespace: str = None
+    ) -> VespaResponse:
         """
         Get a data point from a Vespa app.
 
         :param schema: The schema that we are getting data from.
         :param data_id: Unique id associated with this data point.
+        :param namespace: The namespace that we are getting data from. If no namespace is provided the schema is used.
         :return: Response of the HTTP GET request.
         """
-        with VespaSync(self) as sync_app:
-            return sync_app.get_data(schema=schema, data_id=data_id)
+        if not namespace:
+            namespace = schema
 
-    def _get_batch_sync(self, schema: str, batch: List[Dict]):
-        return [self.get_data(schema, data_point["id"]) for data_point in batch]
+        with VespaSync(self) as sync_app:
+            return sync_app.get_data(
+                schema=schema, data_id=data_id, namespace=namespace
+            )
+
+    def _get_batch_sync(self, schema: str, batch: List[Dict], namespace: str):
+        return [
+            self.get_data(schema, data_point["id"], namespace) for data_point in batch
+        ]
 
     async def _get_batch_async(
-        self, schema: str, batch: List[Dict], connections, total_timeout
+        self, schema: str, batch: List[Dict], connections, total_timeout, namespace: str
     ):
         async with VespaAsync(
             app=self, connections=connections, total_timeout=total_timeout
         ) as async_app:
-            return await async_app.get_batch(schema=schema, batch=batch)
+            return await async_app.get_batch(
+                schema=schema, batch=batch, namespace=namespace
+            )
 
     def get_batch(
         self,
@@ -551,6 +648,7 @@ class Vespa(object):
         asynchronous=True,
         connections: Optional[int] = 100,
         total_timeout: int = 100,
+        namespace: Optional[str] = None,
     ):
         """
         Get a batch of data from a Vespa app.
@@ -561,6 +659,7 @@ class Vespa(object):
         :param asynchronous: Set True to get data in async mode. Default to True.
         :param connections: Number of allowed concurrent connections, valid only if `asynchronous=True`.
         :param total_timeout: Total timeout in secs for each of the concurrent requests when using `asynchronous=True`.
+        :param namespace: The namespace that we are getting data from. If no namespace is provided the schema is used.
         :return: List of HTTP POST responses
         """
         if not schema:
@@ -571,19 +670,28 @@ class Vespa(object):
                     "Not possible to infer schema name. Specify schema parameter."
                 )
 
+        if not namespace:
+            namespace = schema
+
         if asynchronous:
             coro = self._get_batch_async(
                 schema=schema,
                 batch=batch,
                 connections=connections,
                 total_timeout=total_timeout,
+                namespace=namespace,
             )
-            return self._check_for_running_loop_and_run_coroutine(coro=coro)
+            return Vespa._check_for_running_loop_and_run_coroutine(coro=coro)
         else:
-            return self._get_batch_sync(schema=schema, batch=batch)
+            return self._get_batch_sync(schema=schema, batch=batch, namespace=namespace)
 
     def update_data(
-        self, schema: str, data_id: str, fields: Dict, create: bool = False
+        self,
+        schema: str,
+        data_id: str,
+        fields: Dict,
+        create: bool = False,
+        namespace: str = None,
     ) -> VespaResponse:
         """
         Update a data point in a Vespa app.
@@ -592,31 +700,42 @@ class Vespa(object):
         :param data_id: Unique id associated with this data point.
         :param fields: Dict containing all the fields you want to update.
         :param create: If true, updates to non-existent documents will create an empty document to update
+        :param namespace: The namespace that we are updating data. If no namespace is provided the schema is used.
         :return: Response of the HTTP PUT request.
         """
+        if not namespace:
+            namespace = schema
+
         with VespaSync(self) as sync_app:
             return sync_app.update_data(
-                schema=schema, data_id=data_id, fields=fields, create=create
+                schema=schema,
+                data_id=data_id,
+                fields=fields,
+                create=create,
+                namespace=namespace,
             )
 
-    def _update_batch_sync(self, schema: str, batch: List[Dict]):
+    def _update_batch_sync(self, schema: str, batch: List[Dict], namespace: str):
         return [
             self.update_data(
                 schema,
                 data_point["id"],
                 data_point["fields"],
                 data_point.get("create", False),
+                namespace,
             )
             for data_point in batch
         ]
 
     async def _update_batch_async(
-        self, schema: str, batch: List[Dict], connections, total_timeout
+        self, schema: str, batch: List[Dict], connections, total_timeout, namespace: str
     ):
         async with VespaAsync(
             app=self, connections=connections, total_timeout=total_timeout
         ) as async_app:
-            return await async_app.update_batch(schema=schema, batch=batch)
+            return await async_app.update_batch(
+                schema=schema, batch=batch, namespace=namespace
+            )
 
     def update_batch(
         self,
@@ -625,6 +744,7 @@ class Vespa(object):
         asynchronous=True,
         connections: Optional[int] = 100,
         total_timeout: int = 100,
+        namespace: Optional[str] = None,
     ):
         """
         Update a batch of data in a Vespa app.
@@ -635,6 +755,7 @@ class Vespa(object):
         :param asynchronous: Set True to update data in async mode. Default to True.
         :param connections: Number of allowed concurrent connections, valid only if `asynchronous=True`.
         :param total_timeout: Total timeout in secs for each of the concurrent requests when using `asynchronous=True`.
+        :param namespace: The namespace that we are updating data. If no namespace is provided the schema is used.
         :return: List of HTTP POST responses
         """
         if not schema:
@@ -645,16 +766,22 @@ class Vespa(object):
                     "Not possible to infer schema name. Specify schema parameter."
                 )
 
+        if not namespace:
+            namespace = schema
+
         if asynchronous:
             coro = self._update_batch_async(
                 schema=schema,
                 batch=batch,
                 connections=connections,
                 total_timeout=total_timeout,
+                namespace=namespace,
             )
-            return self._check_for_running_loop_and_run_coroutine(coro=coro)
+            return Vespa._check_for_running_loop_and_run_coroutine(coro=coro)
         else:
-            return self._update_batch_sync(schema=schema, batch=batch)
+            return self._update_batch_sync(
+                schema=schema, batch=batch, namespace=namespace
+            )
 
     @staticmethod
     def annotate_data(
@@ -992,7 +1119,7 @@ class Vespa(object):
         model = self.get_model_from_application_package(model_id)
         encoded_tokens = model.create_url_encoded_tokens(x=x)
         with VespaSync(self) as sync_app:
-            return model.parse_vespa_prediction(
+            return TextTask.parse_vespa_prediction(
                 sync_app.predict(
                     model_id=model_id,
                     function_name=function_name,
@@ -1034,19 +1161,6 @@ class VespaSync(object):
             return
         self.http_session.close()
 
-    def get_application_status(self) -> Optional[Response]:
-        """
-        Get application status.
-
-        :return:
-        """
-        end_point = "{}/ApplicationStatus".format(self.app.end_point)
-        try:
-            response = self.http_session.get(end_point, cert=self.cert)
-        except ConnectionError:
-            response = None
-        return response
-
     def get_model_endpoint(self, model_id: Optional[str] = None) -> Optional[dict]:
         """Get model evaluation endpoints."""
         end_point = "{}/model-evaluation/v1/".format(self.app.end_point)
@@ -1084,17 +1198,24 @@ class VespaSync(object):
             response = None
         return response
 
-    def feed_data_point(self, schema: str, data_id: str, fields: Dict) -> VespaResponse:
+    def feed_data_point(
+        self, schema: str, data_id: str, fields: Dict, namespace: str = None
+    ) -> VespaResponse:
         """
         Feed a data point to a Vespa app.
 
         :param schema: The schema that we are sending data to.
         :param data_id: Unique id associated with this data point.
         :param fields: Dict containing all the fields required by the `schema`.
+        :param namespace: The namespace that we are sending data to. If no namespace is provided the schema is used.
         :return: Response of the HTTP POST request.
         """
+
+        if not namespace:
+            namespace = schema
+
         end_point = "{}/document/v1/{}/{}/docid/{}".format(
-            self.app.end_point, schema, schema, str(data_id)
+            self.app.end_point, namespace, schema, str(data_id)
         )
         vespa_format = {"fields": fields}
         response = self.http_session.post(end_point, json=vespa_format, cert=self.cert)
@@ -1129,7 +1250,7 @@ class VespaSync(object):
         :return: Either the request body if debug_request is True or the result from the Vespa application
         """
         body = (
-            self.app._build_query_body(query, query_model, recall, **kwargs)
+            self.app.build_query_body(query, query_model, recall, **kwargs)
             if body is None
             else body
         )
@@ -1145,16 +1266,22 @@ class VespaSync(object):
             json=r.json(), status_code=r.status_code, url=str(r.url)
         )
 
-    def delete_data(self, schema: str, data_id: str) -> VespaResponse:
+    def delete_data(
+        self, schema: str, data_id: str, namespace: str = None
+    ) -> VespaResponse:
         """
         Delete a data point from a Vespa app.
 
         :param schema: The schema that we are deleting data from.
         :param data_id: Unique id associated with this data point.
+        :param namespace: The namespace that we are deleting data from.
         :return: Response of the HTTP DELETE request.
         """
+        if not namespace:
+            namespace = schema
+
         end_point = "{}/document/v1/{}/{}/docid/{}".format(
-            self.app.end_point, schema, schema, str(data_id)
+            self.app.end_point, namespace, schema, str(data_id)
         )
         response = self.http_session.delete(end_point, cert=self.cert)
         return VespaResponse(
@@ -1164,30 +1291,42 @@ class VespaSync(object):
             operation_type="delete",
         )
 
-    def delete_all_docs(self, content_cluster_name: str, schema: str) -> Response:
+    def delete_all_docs(
+        self, content_cluster_name: str, schema: str, namespace: str = None
+    ) -> Response:
         """
         Delete all documents associated with the schema
 
         :param content_cluster_name: Name of content cluster to GET from, or visit.
         :param schema: The schema that we are deleting data from.
+        :param namespace: The namespace that we are deleting data from.
         :return: Response of the HTTP DELETE request.
         """
+        if not namespace:
+            namespace = schema
+
         end_point = "{}/document/v1/{}/{}/docid/?cluster={}&selection=true".format(
-            self.app.end_point, schema, schema, content_cluster_name
+            self.app.end_point, namespace, schema, content_cluster_name
         )
         response = self.http_session.delete(end_point, cert=self.cert)
         return response
 
-    def get_data(self, schema: str, data_id: str) -> VespaResponse:
+    def get_data(
+        self, schema: str, data_id: str, namespace: str = None
+    ) -> VespaResponse:
         """
         Get a data point from a Vespa app.
 
         :param schema: The schema that we are getting data from.
         :param data_id: Unique id associated with this data point.
+        :param namespace: The namespace that we are getting data from.
         :return: Response of the HTTP GET request.
         """
+        if not namespace:
+            namespace = schema
+
         end_point = "{}/document/v1/{}/{}/docid/{}".format(
-            self.app.end_point, schema, schema, str(data_id)
+            self.app.end_point, namespace, schema, str(data_id)
         )
         response = self.http_session.get(end_point, cert=self.cert)
         return VespaResponse(
@@ -1198,7 +1337,12 @@ class VespaSync(object):
         )
 
     def update_data(
-        self, schema: str, data_id: str, fields: Dict, create: bool = False
+        self,
+        schema: str,
+        data_id: str,
+        fields: Dict,
+        create: bool = False,
+        namespace: str = None,
     ) -> VespaResponse:
         """
         Update a data point in a Vespa app.
@@ -1207,10 +1351,14 @@ class VespaSync(object):
         :param data_id: Unique id associated with this data point.
         :param fields: Dict containing all the fields you want to update.
         :param create: If true, updates to non-existent documents will create an empty document to update
+        :param namespace: The namespace that we are updating data.
         :return: Response of the HTTP PUT request.
         """
+        if not namespace:
+            namespace = schema
+
         end_point = "{}/document/v1/{}/{}/docid/{}?create={}".format(
-            self.app.end_point, schema, schema, str(data_id), str(create).lower()
+            self.app.end_point, namespace, schema, str(data_id), str(create).lower()
         )
         vespa_format = {"fields": {k: {"assign": v} for k, v in fields.items()}}
         response = self.http_session.put(end_point, json=vespa_format, cert=self.cert)
@@ -1256,7 +1404,8 @@ class VespaAsync(object):
             return
         return await self.aiohttp_session.close()
 
-    async def _wait(self, f, args, **kwargs):
+    @staticmethod
+    async def _wait(f, args, **kwargs):
         tasks = [asyncio.create_task(f(*arg, **kwargs)) for arg in args]
         await asyncio.wait(tasks, return_when=asyncio.ALL_COMPLETED)
         return [result for result in map(lambda task: task.result(), tasks)]
@@ -1276,7 +1425,7 @@ class VespaAsync(object):
                 body, query, query_model, debug_request, recall, **kwargs
             )
         body = (
-            self.app._build_query_body(query, query_model, recall, **kwargs)
+            self.app.build_query_body(query, query_model, recall, **kwargs)
             if body is None
             else body
         )
@@ -1309,14 +1458,14 @@ class VespaAsync(object):
     ):
         sem = asyncio.Semaphore(self.connections)
         if body_batch:
-            return await self._wait(
+            return await VespaAsync._wait(
                 self._query_semaphore,
                 [(body, None, None, None, sem) for body in body_batch],
                 **kwargs,
             )
         else:
             if recall:
-                return await self._wait(
+                return await VespaAsync._wait(
                     self._query_semaphore,
                     [
                         (None, q, query_model, r, sem)
@@ -1325,7 +1474,7 @@ class VespaAsync(object):
                     **kwargs,
                 )
             else:
-                return await self._wait(
+                return await VespaAsync._wait(
                     self._query_semaphore,
                     [(None, q, query_model, None, sem) for q in query_batch],
                     **kwargs,
@@ -1333,10 +1482,12 @@ class VespaAsync(object):
 
     @retry(wait=wait_exponential(multiplier=1), stop=stop_after_attempt(3))
     async def feed_data_point(
-        self, schema: str, data_id: str, fields: Dict
+        self, schema: str, data_id: str, fields: Dict, namespace: str = None
     ) -> VespaResponse:
+        if not namespace:
+            namespace = schema
         end_point = "{}/document/v1/{}/{}/docid/{}".format(
-            self.app.end_point, schema, schema, str(data_id)
+            self.app.end_point, namespace, schema, str(data_id)
         )
         vespa_format = {"fields": fields}
         response = await self.aiohttp_session.post(end_point, json=vespa_format)
@@ -1348,27 +1499,42 @@ class VespaAsync(object):
         )
 
     async def _feed_data_point_semaphore(
-        self, schema: str, data_id: str, fields: Dict, semaphore: asyncio.Semaphore
+        self,
+        schema: str,
+        data_id: str,
+        fields: Dict,
+        semaphore: asyncio.Semaphore,
+        namespace: str = None,
     ):
+        if not namespace:
+            namespace = schema
+
         async with semaphore:
             return await self.feed_data_point(
-                schema=schema, data_id=data_id, fields=fields
+                schema=schema, data_id=data_id, fields=fields, namespace=namespace
             )
 
-    async def feed_batch(self, schema: str, batch: List[Dict]):
+    async def feed_batch(self, schema: str, batch: List[Dict], namespace=None):
+        if not namespace:
+            namespace = schema
         sem = asyncio.Semaphore(self.connections)
-        return await self._wait(
+        return await VespaAsync._wait(
             self._feed_data_point_semaphore,
             [
-                (schema, data_point["id"], data_point["fields"], sem)
+                (schema, data_point["id"], data_point["fields"], sem, namespace)
                 for data_point in batch
             ],
         )
 
     @retry(wait=wait_exponential(multiplier=1), stop=stop_after_attempt(3))
-    async def delete_data(self, schema: str, data_id: str) -> VespaResponse:
+    async def delete_data(
+        self, schema: str, data_id: str, namespace: str = None
+    ) -> VespaResponse:
+        if not namespace:
+            namespace = schema
+
         end_point = "{}/document/v1/{}/{}/docid/{}".format(
-            self.app.end_point, schema, schema, str(data_id)
+            self.app.end_point, namespace, schema, str(data_id)
         )
         response = await self.aiohttp_session.delete(end_point)
         return VespaResponse(
@@ -1379,22 +1545,37 @@ class VespaAsync(object):
         )
 
     async def _delete_data_semaphore(
-        self, schema: str, data_id: str, semaphore: asyncio.Semaphore
+        self,
+        schema: str,
+        data_id: str,
+        semaphore: asyncio.Semaphore,
+        namespace: str = None,
     ):
-        async with semaphore:
-            return await self.delete_data(schema=schema, data_id=data_id)
+        if not namespace:
+            namespace = schema
 
-    async def delete_batch(self, schema: str, batch: List[Dict]):
+        async with semaphore:
+            return await self.delete_data(
+                schema=schema, data_id=data_id, namespace=namespace
+            )
+
+    async def delete_batch(self, schema: str, batch: List[Dict], namespace: str = None):
         sem = asyncio.Semaphore(self.connections)
-        return await self._wait(
+        if not namespace:
+            namespace = schema
+        return await VespaAsync._wait(
             self._delete_data_semaphore,
-            [(schema, data_point["id"], sem) for data_point in batch],
+            [(schema, data_point["id"], sem, namespace) for data_point in batch],
         )
 
     @retry(wait=wait_exponential(multiplier=1), stop=stop_after_attempt(3))
-    async def get_data(self, schema: str, data_id: str):
+    async def get_data(
+        self, schema: str, data_id: str, namespace: str = None
+    ) -> VespaResponse:
+        if not namespace:
+            namespace = schema
         end_point = "{}/document/v1/{}/{}/docid/{}".format(
-            self.app.end_point, schema, schema, str(data_id)
+            self.app.end_point, namespace, schema, str(data_id)
         )
         response = await self.aiohttp_session.get(end_point)
         return VespaResponse(
@@ -1405,24 +1586,44 @@ class VespaAsync(object):
         )
 
     async def _get_data_semaphore(
-        self, schema: str, data_id: str, semaphore: asyncio.Semaphore
+        self,
+        schema: str,
+        data_id: str,
+        semaphore: asyncio.Semaphore,
+        namespace: str = None,
     ):
+        if not namespace:
+            namespace = schema
+
         async with semaphore:
-            return await self.get_data(schema=schema, data_id=data_id)
+            return await self.get_data(
+                schema=schema, data_id=data_id, namespace=namespace
+            )
+
+    async def get_batch(self, schema: str, batch: List[Dict], namespace: str = None):
+        if not namespace:
+            namespace = schema
 
-    async def get_batch(self, schema: str, batch: List[Dict]):
         sem = asyncio.Semaphore(self.connections)
-        return await self._wait(
+        return await VespaAsync._wait(
             self._get_data_semaphore,
-            [(schema, data_point["id"], sem) for data_point in batch],
+            [(schema, data_point["id"], sem, namespace) for data_point in batch],
         )
 
     @retry(wait=wait_exponential(multiplier=1), stop=stop_after_attempt(3))
     async def update_data(
-        self, schema: str, data_id: str, fields: Dict, create: bool = False
+        self,
+        schema: str,
+        data_id: str,
+        fields: Dict,
+        create: bool = False,
+        namespace: str = None,
     ) -> VespaResponse:
+        if not namespace:
+            namespace = schema
+
         end_point = "{}/document/v1/{}/{}/docid/{}?create={}".format(
-            self.app.end_point, schema, schema, str(data_id), str(create).lower()
+            self.app.end_point, namespace, schema, str(data_id), str(create).lower()
         )
         vespa_format = {"fields": {k: {"assign": v} for k, v in fields.items()}}
         response = await self.aiohttp_session.put(end_point, json=vespa_format)
@@ -1440,15 +1641,25 @@ class VespaAsync(object):
         fields: Dict,
         semaphore: asyncio.Semaphore,
         create: bool = False,
+        namespace: str = None,
     ):
+        if not namespace:
+            namespace = schema
+
         async with semaphore:
             return await self.update_data(
-                schema=schema, data_id=data_id, fields=fields, create=create
+                schema=schema,
+                data_id=data_id,
+                fields=fields,
+                create=create,
+                namespace=namespace,
             )
 
-    async def update_batch(self, schema: str, batch: List[Dict]):
+    async def update_batch(self, schema: str, batch: List[Dict], namespace: str = None):
+        if not namespace:
+            namespace = schema
         sem = asyncio.Semaphore(self.connections)
-        return await self._wait(
+        return await VespaAsync._wait(
             self._update_data_semaphore,
             [
                 (
@@ -1457,6 +1668,7 @@ class VespaAsync(object):
                     data_point["fields"],
                     sem,
                     data_point.get("create", False),
+                    namespace,
                 )
                 for data_point in batch
             ],
