@@ -1,18 +1,18 @@
 import http.client
 import json
 import os
-import re
 import sys
 import zipfile
+import logging
 from base64 import standard_b64encode
 from datetime import datetime, timedelta
 from io import BytesIO
 from pathlib import Path
-from shutil import copyfile
 from time import sleep, strftime, gmtime
 from typing import Union, IO, Optional, Mapping
 
 import docker
+import requests
 from cryptography import x509
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import serialization, hashes
@@ -20,25 +20,27 @@ from cryptography.hazmat.primitives.asymmetric import ec
 
 from vespa.application import Vespa
 from vespa.json_serialization import ToJson, FromJson
-from vespa.package import ApplicationPackage, ModelServer
+from vespa.package import ApplicationPackage
+
+CFG_SERVER_START_TIMEOUT = 300
+APP_INIT_TIMEOUT = 300
 
 
 class VespaDocker(ToJson, FromJson["VespaDocker"]):
     def __init__(
         self,
-        disk_folder: Optional[str] = None,
         port: int = 8080,
         container_memory: Union[str, int] = 4 * (1024 ** 3),
         output_file: IO = sys.stdout,
         container: Optional[docker.models.containers.Container] = None,
-        container_image: str = "vespaengine/vespa"
+        container_image: str = "vespaengine/vespa",
+        cfgsrv_port: int = 19071
     ) -> None:
         """
         Manage Docker deployments.
 
-        :param disk_folder: Disk folder to save the required Vespa config files. Default to application name
-            folder within user's current working directory.
         :param port: Container port.
+        :param cfgsrv_port: Config Server port.
         :param output_file: Output file to write output messages.
         :param container_memory: Docker container memory available to the application.
         :param container: Used when instantiating VespaDocker from a running container.
@@ -54,11 +56,14 @@ class VespaDocker(ToJson, FromJson["VespaDocker"]):
         self.container_id = container_id
         self.url = "http://localhost"
         self.local_port = port
-        self.disk_folder = disk_folder
+        self.cfgsrv_port = cfgsrv_port
         self.container_memory = container_memory
         self.output = output_file
         self.container_image = container_image
 
+        if os.getenv("PYVESPA_DEBUG") == "true":
+            logging.basicConfig(level=logging.DEBUG)
+
     @staticmethod
     def from_container_name_or_id(
         name_or_id: str, output_file: IO = sys.stdout
@@ -68,6 +73,7 @@ class VespaDocker(ToJson, FromJson["VespaDocker"]):
 
         :param name_or_id: Name or id of the container.
         :param output_file: Output file to write output messages.
+        :raises ValueError: Exception if container not found
         :return: VespaDocker instance associated with the running container.
         """
         client = docker.from_env()
@@ -75,7 +81,6 @@ class VespaDocker(ToJson, FromJson["VespaDocker"]):
             container = client.containers.get(name_or_id)
         except docker.errors.NotFound:
             raise ValueError("The container does not exist.")
-        disk_folder = container.attrs["Mounts"][0]["Source"]
         port = int(
             container.attrs["HostConfig"]["PortBindings"]["8080/tcp"][0]["HostPort"]
         )
@@ -83,7 +88,6 @@ class VespaDocker(ToJson, FromJson["VespaDocker"]):
         container_image = container.image.tags[0]  # vespaengine/vespa:latest
 
         return VespaDocker(
-            disk_folder=disk_folder,
             port=port,
             container_memory=container_memory,
             output_file=output_file,
@@ -94,15 +98,26 @@ class VespaDocker(ToJson, FromJson["VespaDocker"]):
     def _run_vespa_engine_container(
         self,
         application_name: str,
-        disk_folder: str,
         container_memory: str,
     ):
         client = docker.from_env()
         if self.container is None:
             try:
+                logging.debug("Try Docker container restart")
                 self.container = client.containers.get(application_name)
                 self.container.restart()
             except docker.errors.NotFound:
+                logging.debug("Start a Docker container: "
+                              "image: {image}, "
+                              "mem_limit: {mem_limit}, "
+                              "name: {name}, "
+                              "hostname: {hostname}, "
+                              "ports: {ports}".format(
+                    image=self.container_image,
+                    mem_limit=container_memory,
+                    name=application_name,
+                    hostname=application_name,
+                    ports={8080: self.local_port, 19071: self.cfgsrv_port}))
                 self.container = client.containers.run(
                     self.container_image,
                     detach=True,
@@ -110,12 +125,11 @@ class VespaDocker(ToJson, FromJson["VespaDocker"]):
                     name=application_name,
                     hostname=application_name,
                     privileged=True,
-                    volumes={disk_folder: {"bind": "/app", "mode": "rw"}},
-                    ports={8080: self.local_port},
-                )
+                    ports={8080: self.local_port, 19071: self.cfgsrv_port})
             self.container_name = self.container.name
             self.container_id = self.container.id
         else:
+            logging.debug("Try Docker container restart")
             self.container.restart()
 
     def _check_configuration_server(self) -> bool:
@@ -123,139 +137,37 @@ class VespaDocker(ToJson, FromJson["VespaDocker"]):
         Check if configuration server is running and ready for deployment
         :return: True if configuration server is running.
         """
-        return (
-            self.container is not None
-            and self.container.exec_run(
-                "bash -c 'curl -s --head http://localhost:19071/ApplicationStatus'"
-            )
-            .output.decode("utf-8")
-            .split("\r\n")[0]
-            == "HTTP/1.1 200 OK"
-        )
+        if self.container is None:
+            return False
 
-    def export_application_package(
-        self, application_package: Union[ApplicationPackage, ModelServer]
-    ) -> None:
+        output = self.container.exec_run(
+            "bash -c 'curl -s --head http://localhost:19071/ApplicationStatus'"
+        ).output.decode("utf-8")
+        logging.debug("Config Server ApplicationStatus head response: " + output)
+        return output.split("\r\n")[0] == "HTTP/1.1 200 OK"
+
+    def wait_for_config_server_start(self, max_wait):
         """
-        Export application package to disk.
-        :param application_package: Application package to export.
-        :return: None. Application package file will be stored on `disk_folder`.
+        Waits for Config Server to start inside the Docker image
+
+        :param max_wait: Seconds to wait for the application endpoint
+        :raises RuntimeError: Raises runtime error if the config server does not start within max_wait
+        :return:
         """
-        if not self.disk_folder:
-            self.disk_folder = os.path.join(os.getcwd(), application_package.name)
+        try_interval = 5
+        waited = 0
+        while not self._check_configuration_server() and (waited < max_wait):
+            print("Waiting for configuration server, {0}/{1} seconds...".format(waited, max_wait), file=self.output)
+            sleep(try_interval)
+            waited += try_interval
+        if waited >= max_wait:
+            self.dump_vespa_log()
+            raise RuntimeError("Config server did not start, waited for {0} seconds.".format(max_wait))
 
-        Path(os.path.join(self.disk_folder, "application/schemas")).mkdir(
-            parents=True, exist_ok=True
-        )
-        Path(os.path.join(self.disk_folder, "application/files")).mkdir(
-            parents=True, exist_ok=True
-        )
-        Path(os.path.join(self.disk_folder, "application/models")).mkdir(
-            parents=True, exist_ok=True
-        )
-
-        for schema in application_package.schemas:
-            with open(
-                os.path.join(
-                    self.disk_folder,
-                    "application/schemas/{}.sd".format(schema.name),
-                ),
-                "w",
-            ) as f:
-                f.write(schema.schema_to_text)
-            for model in schema.models:
-                copyfile(
-                    model.model_file_path,
-                    os.path.join(
-                        self.disk_folder, "application/files", model.model_file_name
-                    ),
-                )
-
-        if application_package.query_profile:
-            Path(
-                os.path.join(
-                    self.disk_folder, "application/search/query-profiles/types"
-                )
-            ).mkdir(parents=True, exist_ok=True)
-            with open(
-                os.path.join(
-                    self.disk_folder,
-                    "application/search/query-profiles/default.xml",
-                ),
-                "w",
-            ) as f:
-                f.write(application_package.query_profile_to_text)
-            with open(
-                os.path.join(
-                    self.disk_folder,
-                    "application/search/query-profiles/types/root.xml",
-                ),
-                "w",
-            ) as f:
-                f.write(application_package.query_profile_type_to_text)
-        with open(os.path.join(self.disk_folder, "application/hosts.xml"), "w") as f:
-            f.write(application_package.hosts_to_text)
-        with open(os.path.join(self.disk_folder, "application/services.xml"), "w") as f:
-            f.write(application_package.services_to_text)
-
-        if application_package.models:
-            for model_id, model in application_package.models.items():
-                model.export_to_onnx(
-                    output_path=os.path.join(
-                        self.disk_folder,
-                        "application/models/{}.onnx".format(model_id),
-                    )
-                )
-
-    def _execute_deployment(
-        self,
-        application_name: str,
-        disk_folder: str,
-        container_memory: str = "4G",
-        application_folder: Optional[str] = None,
-        application_package: Optional[ApplicationPackage] = None,
-    ):
-
-        self._run_vespa_engine_container(
-            application_name=application_name,
-            disk_folder=disk_folder,
-            container_memory=container_memory,
-        )
-
-        while not self._check_configuration_server():
-            print("Waiting for configuration server.", file=self.output)
-            sleep(5)
-
-        _application_folder = "/app"
-        if application_folder:
-            _application_folder = (
-                _application_folder + "/" + application_folder
-            )  # using os.path.join break on windows
-        deployment = self.container.exec_run(
-            "bash -c '/opt/vespa/bin/vespa-deploy prepare {} && /opt/vespa/bin/vespa-deploy activate'".format(
-                _application_folder
-            )
-        )
-
-        deployment_message = deployment.output.decode("utf-8").split("\n")
-
-        if not any(re.match("Generation: [0-9]+", line) for line in deployment_message):
-            raise RuntimeError(deployment_message)
-
-        app = Vespa(
-            url=self.url,
-            port=self.local_port,
-            deployment_message=deployment_message,
-            application_package=application_package,
-        )
-
-        while not app.get_application_status():
-            print("Waiting for application status.", file=self.output)
-            sleep(10)
-
-        print("Finished deployment.", file=self.output)
-
-        return app
+    def dump_vespa_log(self):
+        log_dump = self.container.exec_run("bash -c 'cat /opt/vespa/logs/vespa/vespa.log'")
+        logging.debug("Dumping vespa.log:")
+        logging.debug(log_dump.output.decode("utf-8"))
 
     def deploy(
         self,
@@ -266,41 +178,68 @@ class VespaDocker(ToJson, FromJson["VespaDocker"]):
         :param application_package: ApplicationPackage to be deployed.
         :return: a Vespa connection instance.
         """
-        if not self.disk_folder:
-            self.disk_folder = os.path.join(os.getcwd(), application_package.name)
-
-        self.export_application_package(application_package=application_package)
-
-        return self._execute_deployment(
-            application_name=application_package.name,
-            disk_folder=self.disk_folder,
-            container_memory=self.container_memory,
-            application_folder="application",
-            application_package=application_package,
-        )
+        return self._deploy_data(application_package, application_package.to_zip())
 
     def deploy_from_disk(
         self,
         application_name: str,
-        application_folder: Optional[str] = None,
+        application_root: Path
     ) -> Vespa:
         """
-        Deploy disk-based application package into a Vespa container.
+        Deploy from a directory tree.
+        Used when making changes to application package files not supported by pyvespa -
+        this is why this method is not found in the ApplicationPackage class.
 
-        :param application_name: Name of the application.
-        :param application_folder: Relative path to the folder inside `disk_folder` containing the application files.
-            If None, we assume `disk_folder` to be the application folder.
+        :param application_name: Application package name.
+        :param application_root: Application package directory root
         :return: a Vespa connection instance.
         """
-        if not self.disk_folder:
-            self.disk_folder = os.path.join(os.getcwd(), application_name)
+        tmp_zip = "tmp_app_package.zip"
+        orig_dir = os.getcwd()
+        zipf = zipfile.ZipFile(tmp_zip, "w", zipfile.ZIP_DEFLATED)
+        os.chdir(application_root)  # Workaround to avoid the top-level directory
+        for root, dirs, files in os.walk("."):
+            for file in files:
+                zipf.write(os.path.join(root, file))
+        zipf.close()
+        os.chdir(orig_dir)
+        with open(tmp_zip, "rb") as f:
+            data = f.read()
+        os.remove(tmp_zip)
+        return self._deploy_data(ApplicationPackage(name=application_name), data)
 
-        return self._execute_deployment(
-            application_name=application_name,
-            disk_folder=self.disk_folder,
-            container_memory=self.container_memory,
-            application_folder=application_folder,
+    def _deploy_data(self, application: ApplicationPackage, data) -> Vespa:
+        """
+        Deploys an Application Package as zipped data
+
+        :param application: Application package
+        :raises RuntimeError: Exception if deployment fails
+        :return: A Vespa connection instance
+        """
+        self._run_vespa_engine_container(
+            application_name=application.name,
+            container_memory=self.container_memory
         )
+        self.wait_for_config_server_start(max_wait=CFG_SERVER_START_TIMEOUT)
+
+        r = requests.post("http://localhost:{}/application/v2/tenant/default/prepareandactivate".format(self.cfgsrv_port),
+                          headers={"Content-Type": "application/zip"},
+                          data=data,
+                          verify=False)
+        logging.debug("Deploy status code: {}".format(r.status_code))
+        if r.status_code != 200:
+            raise RuntimeError("Deployment failed, code: {}, message: {}".format(r.status_code,
+                                                                                 json.loads(r.content.decode('utf8'))))
+
+        app = Vespa(
+            url=self.url,
+            port=self.local_port,
+            application_package=application
+        )
+        app.wait_for_application_up(max_wait=APP_INIT_TIMEOUT)
+
+        print("Finished deployment.", file=self.output)
+        return app
 
     def stop_services(self):
         """
@@ -344,9 +283,7 @@ class VespaDocker(ToJson, FromJson["VespaDocker"]):
                 url=self.url,
                 port=self.local_port,
             )
-            while not app.get_application_status():
-                print("Waiting for application status.", file=self.output)
-                sleep(10)
+            app.wait_for_application_up(max_wait=1000000) # wait indefinitely...
             for line in start_services.output.decode("utf-8").split("\n"):
                 print(line, file=self.output)
         else:
@@ -383,7 +320,6 @@ class VespaDocker(ToJson, FromJson["VespaDocker"]):
                 "Unable to instantiate VespaDocker from a running container. Starting new container."
             )
         vespa_docker = VespaDocker(
-            disk_folder=mapping["disk_folder"],
             port=mapping["port"],
             container_memory=mapping["container_memory"],
             container_image=mapping["container_image"],
@@ -392,16 +328,14 @@ class VespaDocker(ToJson, FromJson["VespaDocker"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {
+        return {
             "container_id": self.container_id,
             "container_name": self.container_name,
             "url": self.url,
             "port": self.local_port,
-            "disk_folder": self.disk_folder,
             "container_memory": self.container_memory,
             "container_image": self.container_image,
         }
-        return map
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -411,15 +345,13 @@ class VespaDocker(ToJson, FromJson["VespaDocker"]):
             and self.container_name == other.container_name
             and self.url == other.url
             and self.local_port == other.local_port
-            and self.disk_folder == other.disk_folder
             and self.container_memory == other.container_memory
             and self.container_image.split(":")[0] == other.container_image.split(":")[0]
         )
 
     def __repr__(self):
-        return "{0}({1}, {2}, {3}, {4}, {5}, {6}, {7})".format(
+        return "{0}({1}, {2}, {3}, {4}, {5}, {6})".format(
             self.__class__.__name__,
-            repr(self.disk_folder),
             repr(self.url),
             repr(self.local_port),
             repr(self.container_name),
@@ -730,12 +662,14 @@ class VespaCloud(object):
         run = self._start_deployment(instance, job, disk_folder)
         self._follow_deployment(instance, job, run)
         endpoint_url = self._get_endpoint(instance=instance, region=region)
-        print("Finished deployment.", file=self.output)
-        return Vespa(
+        app = Vespa(
             url=endpoint_url,
             cert=os.path.join(disk_folder, self.private_cert_file_name),
             application_package=self.application_package
         )
+        app.wait_for_application_up(max_wait=APP_INIT_TIMEOUT)
+        print("Finished deployment.", file=self.output)
+        return app
 
     def delete(self, instance: str):
         """
