@@ -4,6 +4,15 @@ from typing import List, Optional, Union, Mapping, Dict, IO
 from pathlib import Path
 from urllib.parse import urlencode
 
+from vespa.package import (
+    ModelConfig,
+    Task,
+    OnnxModel,
+    QueryTypeField,
+    Field,
+    Function,
+    RankProfile,
+)
 from vespa.json_serialization import ToJson, FromJson
 
 #
@@ -16,14 +25,14 @@ try:
         AutoModelForSequenceClassification,
         BertForSequenceClassification,
         BertTokenizerFast,
-        Pipeline,
+        pipeline,
     )
     from transformers.convert_graph_to_onnx import convert_pytorch
 except ModuleNotFoundError:
     raise Exception("Use pip install pyvespa[ml] to install ml dependencies.")
 
 
-class TextTask(object):
+class TextTask(Task):
     def __init__(
         self,
         model_id: str,
@@ -39,7 +48,7 @@ class TextTask(object):
         :param tokenizer: Id of the tokenizer as used by the model hub.
         :param output_file: Output file to write output messages.
         """
-        self.model_id = model_id
+        super().__init__(model_id=model_id)
         self.model = model
         self.tokenizer = tokenizer
         if not self.tokenizer:
@@ -60,8 +69,14 @@ class TextTask(object):
             self._model = AutoModelForSequenceClassification.from_pretrained(self.model)
             print("Model loaded.", file=self.output)
 
-        pipeline = Pipeline(model=self._model, tokenizer=self._tokenizer)
-        return pipeline
+        _pipeline = pipeline(
+            task="text-classification",
+            model=self._model,
+            tokenizer=self._tokenizer,
+            return_all_scores=True,
+            function_to_apply="None",
+        )
+        return _pipeline
 
     def export_to_onnx(self, output_path: str) -> None:
         """
@@ -83,9 +98,11 @@ class TextTask(object):
         :return: list with predictions
         """
         pipeline = self._create_pipeline()
-        return pipeline(text).tolist()[0]
+        predictions = pipeline(text)[0]
+        return [x["score"] for x in predictions]
 
-    def parse_vespa_prediction(self, prediction):
+    @staticmethod
+    def parse_vespa_prediction(prediction):
         return [cell["value"] for cell in prediction["cells"]]
 
     def create_url_encoded_tokens(self, x):
@@ -103,9 +120,16 @@ class SequenceClassification(TextTask):
         """
         Sequence Classification task.
 
+        It takes a text input and returns an array of floats depending on which
+        model is used to solve the task.
+
         :param model_id: Id used to identify the model on Vespa applications.
-        :param model: Id of the model as used by the model hub.
-        :param tokenizer: Id of the tokenizer as used by the model hub.
+        :param model: Id of the model as used by the model hub. Alternatively, it can
+            also be the path to the folder containing the model files, as long as
+            the model config is also there.
+        :param tokenizer: Id of the tokenizer as used by the model hub. Alternatively, it can
+            also be the path to the folder containing the tokenizer files, as long as
+            the model config is also there.
         :param output_file: Output file to write output messages.
         """
         super().__init__(
@@ -131,11 +155,6 @@ class SequenceClassification(TextTask):
         return encoded_tokens
 
 
-class ModelConfig(object):
-    def __init__(self, model_id) -> None:
-        self.model_id = model_id
-
-
 class BertModelConfig(ModelConfig, ToJson, FromJson["BertModelConfig"]):
     def __init__(
         self,
@@ -297,6 +316,7 @@ class BertModelConfig(ModelConfig, ToJson, FromJson["BertModelConfig"]):
 
         :param queries: A List of query texts.
         :param docs: A List of document texts.
+        :param return_tensors: Return tensors
         :return: Dict containing `input_ids`, `token_type_ids` and `attention_mask` encodings.
         """
         query_input_ids = self._query_input_ids(queries=queries)
@@ -354,13 +374,129 @@ class BertModelConfig(ModelConfig, ToJson, FromJson["BertModelConfig"]):
         """
 
         if self._model:
-            pipeline = Pipeline(model=self._model, tokenizer=self._tokenizer)
-            convert_pytorch(
-                pipeline, opset=11, output=Path(output_path), use_external_format=False
+            _pipeline = pipeline(
+                task="text-classification",
+                model=self._model,
+                tokenizer=self._tokenizer,
+                return_all_scores=True,
+                function_to_apply="None",
             )
+            convert_pytorch(_pipeline, opset=11, output=Path(output_path), use_external_format=False)
         else:
             raise ValueError("No BERT model found to be exported.")
 
+    def onnx_model(self):
+        model_file_path = self.model_id + ".onnx"
+        self.export_to_onnx(output_path=model_file_path)
+
+        return OnnxModel(
+            model_name=self.model_id,
+            model_file_path=model_file_path,
+            inputs={
+                "input_ids": "input_ids",
+                "token_type_ids": "token_type_ids",
+                "attention_mask": "attention_mask",
+            },
+            outputs={"output_0": "logits"},
+        )
+
+    def query_profile_type_fields(self):
+        return [
+            QueryTypeField(
+                name="ranking.features.query({})".format(self.query_token_ids_name),
+                type="tensor<float>(d0[{}])".format(int(self.actual_query_input_size)),
+            )
+        ]
+
+    def document_fields(self, document_field_indexing):
+        if not document_field_indexing:
+            document_field_indexing = ["attribute", "summary"]
+
+        return [
+            Field(
+                name=self.doc_token_ids_name,
+                type="tensor<float>(d0[{}])".format(int(self.actual_doc_input_size)),
+                indexing=document_field_indexing,
+            ),
+        ]
+
+    def rank_profile(self, include_model_summary_features, **kwargs):
+        constants = {"TOKEN_NONE": 0, "TOKEN_CLS": 101, "TOKEN_SEP": 102}
+        # ToDo: Add a unit test for constants
+        if "constants" in kwargs:
+            constants.update(kwargs.pop("constants"))
+
+        functions = [
+            Function(
+                name="question_length",
+                expression="sum(map(query({}), f(a)(a > 0)))".format(
+                    self.query_token_ids_name
+                ),
+            ),
+            Function(
+                name="doc_length",
+                expression="sum(map(attribute({}), f(a)(a > 0)))".format(
+                    self.doc_token_ids_name
+                ),
+            ),
+            Function(
+                name="input_ids",
+                expression="tokenInputIds({}, query({}), attribute({}))".format(
+                    self.input_size,
+                    self.query_token_ids_name,
+                    self.doc_token_ids_name,
+                ),
+            ),
+            Function(
+                name="attention_mask",
+                expression="tokenAttentionMask({}, query({}), attribute({}))".format(
+                    self.input_size,
+                    self.query_token_ids_name,
+                    self.doc_token_ids_name,
+                ),
+            ),
+            Function(
+                name="token_type_ids",
+                expression="tokenTypeIds({}, query({}), attribute({}))".format(
+                    self.input_size,
+                    self.query_token_ids_name,
+                    self.doc_token_ids_name,
+                ),
+            ),
+            Function(
+                name="logit0",
+                expression="onnx(" + self.model_id + ").logits{d0:0,d1:0}",
+            ),
+            Function(
+                name="logit1",
+                expression="onnx(" + self.model_id + ").logits{d0:0,d1:1}",
+            ),
+        ]
+        if "functions" in kwargs:
+            functions.extend(kwargs.pop("functions"))
+
+        summary_features = []
+        if include_model_summary_features:
+            summary_features.extend(
+                [
+                    "logit0",
+                    "logit1",
+                    "input_ids",
+                    "attention_mask",
+                    "token_type_ids",
+                ]
+            )
+        if "summary_features" in kwargs:
+            summary_features.extend(kwargs.pop("summary_features"))
+
+        return RankProfile(
+            name=self.model_id,
+            constants=constants,
+            functions=functions,
+            summary_features=summary_features,
+            **kwargs
+        )
+
     @staticmethod
     def from_dict(mapping: Mapping) -> "BertModelConfig":
         return BertModelConfig(
@@ -373,14 +509,13 @@ class BertModelConfig(ModelConfig, ToJson, FromJson["BertModelConfig"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {
+        return {
             "model_id": self.model_id,
             "query_input_size": self.query_input_size,
             "doc_input_size": self.doc_input_size,
             "tokenizer": self.tokenizer,
             "model": self.model,
         }
-        return map
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
