@@ -17,6 +17,8 @@ from vespa.test_integration_docker import (
     create_sequence_classification_task,
 )
 
+CFG_SERVER_START_TIMEOUT = 300
+APP_INIT_TIMEOUT = 300
 
 class TestVespaKeyAndCertificate(unittest.TestCase):
     def setUp(self) -> None:
@@ -57,6 +59,7 @@ class TestVespaKeyAndCertificate(unittest.TestCase):
             cert=os.path.join(self.disk_folder, "cert_file.txt"),
             application_package=self.app.application_package,
         )
+        self.app.wait_for_application_up(max_wait=APP_INIT_TIMEOUT)
         self.assertEqual(200, self.app.get_application_status().status_code)
         self.assertDictEqual(
             {
@@ -104,7 +107,6 @@ class TestMsmarcoApplication(TestApplicationCommon):
         ]
 
     def test_model_endpoints_when_no_model_is_available(self):
-        # The port should not be 4443, see https://jira.vzbuilders.com/browse/VESPA-21365
         self.get_model_endpoints_when_no_model_is_available(
             app=self.app,
             expected_model_endpoint="https://{}-container.{}.pyvespa-integration.vespa-team.aws-us-east-1c.dev.z.vespa-app.cloud:4443/model-evaluation/v1/".format(
@@ -227,7 +229,6 @@ class TestCord19Application(TestApplicationCommon):
         ]
 
     def test_model_endpoints_when_no_model_is_available(self):
-        # The port should not be 4443, see https://jira.vzbuilders.com/browse/VESPA-21365
         self.get_model_endpoints_when_no_model_is_available(
             app=self.app,
             expected_model_endpoint="https://{}-container.{}.pyvespa-integration.vespa-team.aws-us-east-1c.dev.z.vespa-app.cloud:4443/model-evaluation/v1/".format(
@@ -351,7 +352,6 @@ class TestQaApplication(TestApplicationCommon):
         ]
 
     def test_model_endpoints_when_no_model_is_available(self):
-        # The port should not be 4443, see https://jira.vzbuilders.com/browse/VESPA-21365
         self.get_model_endpoints_when_no_model_is_available(
             app=self.app,
             expected_model_endpoint="https://{}-container.{}.pyvespa-integration.vespa-team.aws-us-east-1c.dev.z.vespa-app.cloud:4443/model-evaluation/v1/".format(
